-- watches

ALTER TABLE `watches` RENAME TO `watches_old`;

CREATE TABLE `watches` (
  `_id`                      INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
  `serial_number`            TEXT NOT NULL,
  `minute_samples_cursor`    INTEGER NOT NULL DEFAULT 0,
  `activity_sessions_cursor` INTEGER NOT NULL DEFAULT 0
);

CREATE UNIQUE INDEX `watches_serial_number` ON `watches` (`serial_number`);

INSERT INTO `watches`
  SELECT
    NULL,
    `serial_number`,
    COALESCE(`steps_cursor`, 0),
    COALESCE(`activity_sessions_cursor`, 0)
  FROM `watches_old`;

DROP TABLE `watches_old`;

-- ensure watches exist

INSERT INTO watches
  SELECT
    NULL,
    `watch_serial`,
    COALESCE(MAX(`minute_samples_cursor`), 0),
    COALESCE(MAX(`activity_sessions_counter`), 0)
  FROM (
      SELECT
        `s`.`watch_serial`,
        MAX(`s`.`id`) AS `minute_samples_cursor`,
        NULL as `activity_sessions_counter`
      FROM
        `steps` `s`
      GROUP BY
        `s`.`watch_serial`
    UNION
      SELECT
        `a`.`watch_serial`,
        NULL AS `minute_samples_cursor`,
        MAX(`a`.`id`) AS `activity_sessions_counter`
      FROM
        `activity_sessions` `a`
      GROUP BY
        `a`.`watch_serial`
  )
  WHERE
    `watch_serial` NOT IN (
      SELECT
        `serial_number`
      FROM
        `watches`
    )
  GROUP BY
    `watch_serial`;

-- minute_samples

CREATE TABLE `minute_samples` (
  `_id`                 INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
  `watch_id`            INTEGER NOT NULL,
  `date_utc_secs`       INTEGER NOT NULL,
  `utc_to_local_secs`   INTEGER NOT NULL DEFAULT 0,
  `date_local_secs`     INTEGER NOT NULL, -- date_utc_secs + utc_to_local_secs
  `plugged_in`          INTEGER NOT NULL DEFAULT 0,
  `vmc`                 INTEGER NOT NULL DEFAULT 0,
  `light`               INTEGER NOT NULL DEFAULT 0,
  `orientation`         INTEGER NOT NULL DEFAULT 0,
  `step_count`          INTEGER NOT NULL DEFAULT 0,
  `distance_mm`         INTEGER NOT NULL DEFAULT 0,
  `resting_gcal`        INTEGER NOT NULL DEFAULT 0,
  `active_gcal`         INTEGER NOT NULL DEFAULT 0,
  `active_minutes`      INTEGER NOT NULL DEFAULT 0,
  `heart_rate_bpm`      INTEGER,
  `heart_rate_weight`   INTEGER,
  `heart_rate_zone`     INTEGER
);

CREATE UNIQUE INDEX `minute_samples_uniq`     ON `minute_samples` (`watch_id`, `date_utc_secs`);
CREATE INDEX `minute_samples_date_local_secs` ON `minute_samples` (`date_local_secs`);

INSERT INTO `minute_samples`
  SELECT
    `id`,
    (SELECT `_id` FROM `watches` WHERE `serial_number` = `watch_serial`),
    `date`,
    `tz_offset`,
    `date_tz`,
    `plugged_in`,
    `vmc`,
    `light`,
    `orientation`,
    `step_count`,
    `distance` * 10,
    `calories`,
    `active_calories`,
    `time_active`,
    `heart_rate_bpm`,
    `heart_rate_weight`,
    NULL
  FROM `steps`
;

DROP TABLE `steps`;

-- activity_sessions

ALTER TABLE `activity_sessions` RENAME TO `activity_sessions_old`;

CREATE TABLE `activity_sessions` (
  `_id`                 INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
  `watch_id`            INTEGER NOT NULL,
  `type`                INTEGER NOT NULL,
  `start_utc_secs`      INTEGER NOT NULL,
  `end_utc_secs`        INTEGER NOT NULL,
  `utc_to_local_secs`   INTEGER NOT NULL DEFAULT 0,
  `start_local_secs`    INTEGER NOT NULL, -- start_utc_secs + utc_to_local_secs
  `end_local_secs`      INTEGER NOT NULL, -- end_utc_secs + utc_to_local_secs
  `step_count`          INTEGER NOT NULL DEFAULT 0,
  `distance_mm`         INTEGER NOT NULL DEFAULT 0,
  `resting_gcal`        INTEGER NOT NULL DEFAULT 0,
  `active_gcal`         INTEGER NOT NULL DEFAULT 0
);

CREATE UNIQUE INDEX `activity_sessions_uniq`      ON `activity_sessions` (`watch_id`, `type`, `start_utc_secs`);

CREATE INDEX `activity_sessions_start_local_secs` ON `activity_sessions` (`start_local_secs`);
CREATE INDEX `activity_sessions_end_local_secs`   ON `activity_sessions` (`end_local_secs`);

INSERT INTO `activity_sessions`
  SELECT
    `id`,
    (SELECT `_id` FROM `watches` WHERE `serial_number` = `watch_serial`),
    `activity_type`,
    `start_time`,
    `end_time`,
    `tz_offset`,
    `start_date_tz`,
    `end_date_tz`,
    `step_count`,
    `distance` * 1000,
    `calories` * 1000,
    `active_calories` * 1000
  FROM `activity_sessions_old`
;

DROP TABLE `activity_sessions_old`;

DROP TABLE `heart_rates`;
